package cq.flink.kafka.mqtt;


import cq.flink.kafka.mqtt.cfg.ApplicationCfg;
import cq.flink.kafka.mqtt.cfg.JobDescriptor;
import cq.flink.kafka.mqtt.cfg.JsonMapper;
import cq.flink.kafka.mqtt.funcs.LogFilterMapFunction;
import cq.flink.kafka.mqtt.sink.MqttSink;
import cq.flink.kafka.mqtt.source.KafkaSourceCreator;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

@Slf4j
public class KafkaToMqttApplication {

    public static void main(String[] args) throws Exception{
        if (args.length != 1) {
            System.err.println("Usage: KafkaToMqttApplication <config file>");
            log.error("Usage: KafkaToMqttApplication <config file>");
            System.exit(1);
        }
        ApplicationCfg cfg = null;
        try {
            cfg = JsonMapper.MAPPER.readValue(readFileToString(args[0]), ApplicationCfg.class);
        } catch (IOException e) {
            log.error("Error reading config file: " + args[0], e);
            System.exit(1);
        }
        
        startFlinkJob(cfg);
    }
    
    private static void startFlinkJob(ApplicationCfg cfg) throws Exception{
        if (cfg == null) {
            throw new RuntimeException("Application Config is null. 配置文件为空!");
        }
        if (cfg.getMqtt() == null) {
            throw new RuntimeException("Mqtt Config is null. Mqtt配置为空!");
        }
        JobDescriptor jobDescriptor = cfg.getJob() == null ? JobDescriptor.createDefault() : cfg.getJob();
        log.info("Use JobDescriptor: [{}]", jobDescriptor);
        log.info("Use ApplicationCfg: \n{}", JsonMapper.MAPPER.writerWithDefaultPrettyPrinter().writeValueAsString(cfg));
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        env.setParallelism(jobDescriptor.getParallelism());
        env.fromSource(KafkaSourceCreator.createSourceOfString(cfg),
                        WatermarkStrategy.noWatermarks(), jobDescriptor.getSourceName())
                .flatMap(new LogFilterMapFunction())
                .addSink(new MqttSink(cfg.getMqtt().getHost(), cfg.getMqtt().getPort(),
                        cfg.getMqtt().getMaxInflightQueue(), cfg.getMqtt().getMaxMessageSize()))
                .name(jobDescriptor.getSinkName());

        env.execute(jobDescriptor.getJobName());
    }

    /**
     * 读取文件内容
     * @param filepath 文件路径
     * @return 文件内容
     */
    private static String readFileToString(String filepath) throws IOException {
        StringBuilder content = new StringBuilder();
        try (BufferedReader br = new BufferedReader(new FileReader(filepath))) {
            String line;
            while ((line = br.readLine()) != null) {
                content.append(line).append("\n");
            }
        } catch (IOException e) {
            throw new IOException("Error reading file: " + filepath, e);
        }
        return content.toString();
    }
}
