package cq.flink.kafka.mqtt.sink;

import io.netty.handler.codec.mqtt.MqttQoS;
import io.vertx.core.buffer.Buffer;
import io.vertx.mqtt.MqttClient;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;

import java.util.concurrent.atomic.AtomicBoolean;

@Slf4j
public class MqttSink extends RichSinkFunction<Tuple2<String, String>> {

    private static final long serialVersionUID = 7883296716671354462L;

    private transient MqttClient client;
    private AtomicBoolean connecting = new AtomicBoolean(false);
    

    private final String host;
    private final int port;
    private final int maxInflightQueue;
    private final int maxMessageSize;

    public MqttSink(String host, int port, int maxInflightQueue, int maxMessageSize) {
        this.host = host;
        this.port = port;
        this.maxInflightQueue = maxInflightQueue;
        this.maxMessageSize = maxMessageSize;
    }


    @Override
    public void open(Configuration parameters) throws Exception {
        connect();
    }

    private void connect() {
        if (client == null) {
            initMqttClient();
        }
        if (this.connecting == null) {
            this.connecting = new AtomicBoolean(false);
        }
        if (connecting.getAndSet(true)) {
            return;
        }
        if (!client.isConnected()) {
            client.connect(port, host, conn -> {
                try {
                    if (conn.succeeded()) {
                        log.info("Connected!");
                    } else {
                        log.error("Connect failed, retrying after 60s...", conn.cause());
                        MqttClientManager.getVertx().setTimer(60000, id -> connect());
                    }
                } finally {
                    connecting.set(false);
                }
            });
        }
    }

    @Override
    public void invoke(Tuple2<String, String> record, Context context) throws Exception {
        if (client == null || !client.isConnected()) {
            connect();
            return;
        }
        publish(record.f0, record.f1);
    }

    private void initMqttClient() {
        log.info("MQTT Client is null. Creating new client...");
        this.client = MqttClientManager.createClient(maxInflightQueue, maxMessageSize);
    }

    private void publish(String topic, String payload) {
        client.publish(topic, Buffer.buffer(payload), MqttQoS.AT_LEAST_ONCE, false, false, pub -> {
            if (pub.succeeded()) {
                log.info("Published [{}...] to {}", payload.substring(0, 1 << 4), topic);
            } else {
                log.error("Publish failed!", pub.cause());
            }
        });
    }

    @Override
    public void close() {
        if (client != null && client.isConnected()) {
            client.disconnect();
        }
    }

}

